import phonenumbers

class PhoneNumber:
    number: str;
    national_number: str;
    international_number: str;
    dialCode: str;
    country_code: str;

    def __init__(self, number, country_code=None):
        parse = self.parse(number,country_code);
        if parse is not None:
            (
                self.number, self.national_number, 
                self.international_number, self.dialCode,self.country_code
            ) = parse
            try:
                self.parsed = phonenumbers.parse(self.number, self.country_code)
            except:
                pass
    
    def parse(self, _number, _country_code=None):
        try:
            number = _number
            country_code = str(_country_code).upper();
            parsed = phonenumbers.parse(number, country_code);
            national_number = phonenumbers.format_number(parsed, phonenumbers.PhoneNumberFormat.NATIONAL);
            international_number = phonenumbers.format_number(phonenumbers.parse(number, country_code), phonenumbers.PhoneNumberFormat.INTERNATIONAL);
            dialCode = parsed.country_code;
            if str(dialCode).startswith('+') is False:
                dialCode = f"+{dialCode}"
            country_code = phonenumbers.region_code_for_country_code(parsed.country_code)
            return number, national_number,international_number,dialCode,country_code;
        except Exception as e:
            return number, None, None, None, country_code;
    
    def to_json(self, _fields=None):
       if getattr(self, 'number', None) is None:
           return None;
       return {
           "number": self.number,
           "national_number": self.national_number,
           "international_number": self.international_number,
           "dialCode": self.dialCode,
           "country_code": self.country_code
       }

    def is_possible(self):
        try:
            return phonenumbers.is_possible_number(self.parsed)
        except:
            return False;
    
    def is_valid(self):
        try:
            return phonenumbers.is_valid_number(self.parsed)
        except:
            return False;

    def is_valid_for_region(self, region: str = None):
        country_code = region;
        if region is None:
            region = self.country_code;
        
        country_code = str(country_code).upper()
        try:
            return phonenumbers.is_valid_number_for_region(phonenumbers.parse(self.number, country_code), country_code)
        except:
            return False;


from adafri.utils.utils import JsonEncoder, DictUtils, get_object_model_class
from dataclasses import dataclass
import json
import pydash
from firebase_admin.firestore import firestore
from google.cloud.firestore_v1.base_query import FieldFilter, And
import os
from ..auth.firebase_auth import FirestoreApp

def getTimestamp():
    return firestore.SERVER_TIMESTAMP;

@dataclass(init=False)
class FirebaseCollectionBase:
    # _db: firestore.Client = None;
    # _fields_props = None;
    # _collection_name = None;

    def __init__(self, collection_name=None, db: firestore.Client=None, fields=None, fields_props=None, id=None):
        self.collection_name = collection_name;
        self.db = db;
        self.id = id;
        if self.db is None:
            self.db = FirestoreApp().firestore_client()
        self.fields = fields;
        self.fields_props = fields_props;
        
    def print_db(self):
        print('db prtiny',self.db)

    def setFields(self, fields):
        self.fields_props = fields;
    
    def generate_model(self, _key_):
        user = {};
        props = self.fields_props
        for k in DictUtils.get_keys(props):
            user[k] = props[k][_key_];
        return user;

    def collection(self):
        return self.db.collection(self.collection_name)

    def document_reference(self, _id=None) -> 'firestore.DocumentReference':
        id = _id;
        if id is None:
            if self.id is not None:
                id = self.id;
        return self.collection().document(id);

    # def get(self, id):
    #     if id is None or bool(id) is False:
    #          return None;
    #     doc = self.document_reference(id).get();
    #     if doc.exists is False:
    #         return None;
    #     return doc.to_dict();
    
    def update(self, id, data, merge=True):
        if id is None or bool(id) is False:
            return None;
        doc = self.document_reference(id);
        if doc.get().exists is False:
            return None;
        try:
            doc.set(data, merge=merge);
            return data;
        except:
            return None
    
    def remove(self, id):
        if id is None or bool(id) is False:
             return None;
        doc = self.document_reference(id);
        if doc.get().exists is False:
            return None;
        try:
            doc.delete();
            return id;
        except:
            return None

    def custom_query(self, query_params, first=True, limit: int=None):
        i = 0;
        dynamic_query = None;
        filters = []
        while i < len(query_params):
            query = query_params[i];
            
            key = None;
            if 'key' in query:
                key = query['key'];
            
            comparator = None;
            if 'comp' in query:
                comparator = query['comp']

            value = None;
            if 'value' in query:
                value = query['value'];
            if None not in [key, comparator, value]:
                filters.append(FieldFilter(key, comparator, value))
            i+=1;

        and_filter = And(filters=filters)
        dynamic_query = self.collection().where(filter=and_filter)
        if dynamic_query is None:
            return None
        _limit = limit
        if _limit is not None and _limit > 0 or first:
            if _limit is None:
                _limit = 1
            dynamic_query = dynamic_query.limit(_limit)
        
        data = []
        for stream in dynamic_query.stream():
            data.append({"id": stream.id, **stream.to_dict()});
        if first is False:
            if len(data) == 0:
                return []
            return data;
        else:
            if len(data) == 0:
                return None;
            return data[0]
        
    
    
    def to_json(self, _fields=None):
        fields = self.fields;
        if _fields is not None and type(_fields) is list and len(_fields)>0:
            fields = _fields;
        json_object, keys = get_object_model_class({}, self, self.fields_props);
        if fields is None or type(fields) is not list or len(fields)==0:
            return json_object
        return pydash.pick(json_object, fields)
    





from authlib.integrations.flask_oauth2 import (
    AuthorizationServer,
    ResourceProtector,
)
from authlib.oauth2.rfc6749 import grants
from authlib.oauth2.rfc7636 import CodeChallenge

from ...oauth import (OAuthClient, OAuthToken, AuthorizationCodeGrant as AuthCodeGrant, 
                                TokenRevocationEndpoint, TokenValidator, RefreshTokenGrant, TokenGenerator)

class AuthServer(AuthorizationServer):
    OAUTH2_TOKEN_EXPIRES_IN = {
    'authorization_code': 864000,
    'implicit': 3600,
    'password': 864000,
    'client_credentials': 864000
    }
    def authenticate_client(self, request, methods, endpoint='token'):
        return super().authenticate_client(request, methods, endpoint)
    
    def handle_error_response(self, request, error):
        return self.handle_response(*error(self.get_error_uri(request, error)))
    

# authorization_server = AuthServer(
#     query_client=OAuthClient().get_by_client_id,
#     save_token=OAuthToken().save,
# )
authorization_server = AuthorizationServer()
require_oauth = ResourceProtector()

require_oauth = ResourceProtector()

def config_oauth(app, query_client=None, save_token=None, token_generators=[]):
    if query_client is None:
        query_client = OAuthClient().get_by_client_id
    if save_token is None:
        save_token = OAuthToken().save
    authorization_server.query_client = query_client
    authorization_server.save_token = save_token
    authorization_server.init_app(app)
    
    authorization_server.register_grant(grants.ImplicitGrant)
    #authorization_server.register_grant(AccessTokenGrant, [CodeChallenge(required=True)])
    authorization_server.register_grant(grants.ClientCredentialsGrant)
    authorization_server.register_grant(AuthCodeGrant, [CodeChallenge(required=True)])
    
    # authorization_server.register_grant(PasswordGrant)
    authorization_server.register_grant(RefreshTokenGrant)
    # support revocation
    # revocation_cls = create_revocation_endpoint()
    authorization_server.register_endpoint(TokenRevocationEndpoint)
    # for token_generator in token_generators:
    #     type = getattr(token_generator, 'type', None);
    #     generator = getattr(token_generator, 'generator', None);
    #     if None not in [type, generator]:
    #         authorization_server.register_token_generator(type, generator)
    authorization_server.register_token_generator("default", TokenGenerator.generate)
    authorization_server.register_token_generator("client_credentials", TokenGenerator.generate)
    # protect resource
    #bearer_cls = create_bearer_token_validator(TokenValidator)
    require_oauth.register_token_validator(TokenValidator())


# from authlib.jose import JsonWebKey
# from cryptography.hazmat.primitives.asymmetric import rsa
# from cryptography.hazmat.backends import default_backend




# # Step 1: Generate RSA private key
# private_key = rsa.generate_private_key(
#     public_exponent=65537,
#     key_size=2048,
#     backend=default_backend()
# )

# # Step 2: Generate a JWK (JSON Web Key) from the RSA key
# jwk = JsonWebKey().generate_key(kty='RSA', crv_or_size=2048)

# # Convert the private key to JWK format
# private_jwk = jwk.as_dict(is_private=True)
# public_jwk = jwk.as_dict(is_private=False)

# print("Private JWK:", private_jwk)
# print("Public JWK:", public_jwk)
# jwk_set = {
#     "keys": [public_jwk]
# }

# # @app.route('/jwks')
# # def jwks():
# #     return jsonify(jwk_set)

# # Save the keys as needed